@@ -8,8 +8,9 @@ from typing import (
     TYPE_CHECKING, Any, Callable, Concatenate, Generator, Generic, Iterable, Iterator, Mapping,
     NoReturn, Protocol, Sequence, TypeVar, cast, overload
 )
+from typing_extensions import Self
 
-from .builtins import F0, F1, P0, P1, R0, R1, T0, T1, T2, KwargsT, P, R, T
+from .builtins import F0, F1, P0, P1, R0, R1, T0, T1, T2, KwargsT, P, R, R_co, R0_co, T
 
 __all__ = [
     'copy_signature',
@@ -97,10 +98,15 @@ class injected_self_func(Generic[T, P, R], Protocol):  # type: ignore[misc]
         ...
 
 
-self_objects_cache = dict[type[T], T]()  # type: ignore
+self_objects_cache = dict[type, Any]()
 
 
 class inject_self_base(Generic[T, P, R]):
+    cache: bool | None
+    signature: Signature | None
+    init_kwargs: list[str] | None
+    first_key: str | None
+
     def __init__(self, function: Callable[Concatenate[T, P], R], /, *, cache: bool = False) -> None:
         """
         Wrap ``function`` to always have a self provided to it.
@@ -126,40 +132,40 @@ class inject_self_base(Generic[T, P, R]):
     def __get__(
         self, class_obj: type[T] | T | None, class_type: type[T] | type[type[T]]  # type: ignore
     ) -> injected_self_func[T, P, R]:
-        if not self.signature or not self.first_key:  # type: ignore
-            self.signature = Signature.from_callable(self.function, eval_str=True)  # type: ignore
-            self.first_key = next(iter(list(self.signature.parameters.keys())), None)  # type: ignore
+        if not self.signature or not self.first_key:
+            self.signature = Signature.from_callable(self.function, eval_str=True)
+            self.first_key = next(iter(list(self.signature.parameters.keys())), None)
 
             if isinstance(self, inject_self.init_kwargs):
                 from ..exceptions import CustomValueError
 
-                if 4 not in {x.kind for x in self.signature.parameters.values()}:  # type: ignore
+                if 4 not in {x.kind for x in self.signature.parameters.values()}:
                     raise CustomValueError(
                         'This function hasn\'t got any kwargs!', 'inject_self.init_kwargs', self.function
                     )
 
-                self.init_kwargs = list[str](  # type: ignore
-                    k for k, x in self.signature.parameters.items() if x.kind != 4  # type: ignore
+                self.init_kwargs = list[str](
+                    k for k, x in self.signature.parameters.items() if x.kind != 4
                 )
 
         @wraps(self.function)
         def _wrapper(*args: Any, **kwargs: Any) -> Any:
             first_arg = (args[0] if args else None) or (
-                kwargs.get(self.first_key, None) if self.first_key else None  # type: ignore
+                kwargs.get(self.first_key, None) if self.first_key else None
             )
 
             if (
                 first_arg and (
                     (is_obj := isinstance(first_arg, class_type))
-                    or isinstance(first_arg, type(class_type))  # noqa
-                    or first_arg is class_type  # noqa
+                    or isinstance(first_arg, type(class_type))
+                    or first_arg is class_type
                 )
             ):
                 obj = first_arg if is_obj else first_arg()
                 if args:
                     args = args[1:]
                 elif kwargs and self.first_key:
-                    kwargs.pop(self.first_key)  # type: ignore
+                    kwargs.pop(self.first_key)
             elif class_obj is None:
                 if self.cache:
                     if class_type not in self_objects_cache:
@@ -167,7 +173,7 @@ class inject_self_base(Generic[T, P, R]):
                     else:
                         obj = self_objects_cache[class_type]
                 elif self.init_kwargs:
-                    obj = class_type(  # type: ignore
+                    obj = class_type(
                         *self.args, **(self.kwargs | {k: v for k, v in kwargs.items() if k not in self.init_kwargs})
                     )
                     if self.clean_kwargs:
@@ -473,7 +479,7 @@ class classproperty(Generic[P, R, T, T0, P0]):
 
         return self.fget.__get__(__obj, __type)()  # type: ignore
 
-    def __set__(self, __obj: Any, __value: T1) -> None:
+    def __set__(self, __obj: Any, __value: Any) -> None:
         from ..exceptions import CustomError
 
         if not self.fset:
@@ -503,7 +509,7 @@ class classproperty(Generic[P, R, T, T0, P0]):
         return self.fget.__name__
 
 
-class cachedproperty(property, Generic[P, R, T, T0, P0]):
+class cachedproperty(property, Generic[P, R_co, T, T0, P0]):
     """
     Wrapper for a one-time get property, that will be cached.
 
@@ -532,21 +538,29 @@ class cachedproperty(property, Generic[P, R, T, T0, P0]):
 
     if TYPE_CHECKING:
         def __init__(
-            self, fget: Callable[P, R], fset: Callable[[T, T0], None] | None = None,
+            self, fget: Callable[P, R_co], fset: Callable[[T, T0], None] | None = None,
             fdel: Callable[P0, None] | None = None, doc: str | None = None,
         ) -> None:
             ...
 
-        def getter(self, __fget: Callable[P1, R1]) -> cachedproperty[P1, R1, T, T0, P0]:
+        def getter(self, __fget: Callable[P1, R0_co]) -> cachedproperty[P1, R0_co, T, T0, P0]:
             ...
 
-        def setter(self, __fset: Callable[[T1, T2], None]) -> cachedproperty[P, R, T1, T2, P0]:
+        def setter(self, __fset: Callable[[T1, T2], None]) -> cachedproperty[P, R_co, T1, T2, P0]:
             ...
 
-        def deleter(self, __fdel: Callable[P1, None]) -> cachedproperty[P, R, T, T0, P1]:
+        def deleter(self, __fdel: Callable[P1, None]) -> cachedproperty[P, R_co, T, T0, P1]:
             ...
 
-    def __get__(self, __obj: Any, __type: type | None = None) -> R:
+    @overload
+    def __get__(self, __obj: None, __type: type | None = None) -> Self:
+        ...
+
+    @overload
+    def __get__(self, __obj: object, __type: type | None = None) -> R_co:
+        ...
+
+    def __get__(self, __obj: Any, __type: type | None = None) -> Any:
         if isinstance(self.fget, classproperty):
             function = partial(self.fget.__get__, __obj, __type)  # type: ignore
             __obj = __type
@@ -557,14 +571,19 @@ class cachedproperty(property, Generic[P, R, T, T0, P0]):
             cache = getattr(__obj, cachedproperty.cache_key)
             name = self.fget.__name__
         else:
-            function = self.fget.__get__(__obj, __type)  # type: ignore
+            assert self.fget
+            function = self.fget.__get__(__obj, __type)
             cache = __obj.__dict__.get(cachedproperty.cache_key)
             name = function.__name__
 
         if name not in cache:
             cache[name] = function()
 
-        return cache[name]  # type: ignore
+        return cache[name]
+
+    if TYPE_CHECKING:
+        def __set__(self, obj: Any, value: R_co, /) -> None:  # type: ignore[misc]
+            ...
 
 
 class KwargsNotNone(KwargsT):
